# TOOL input-sleep-output.R: InputSleepOutput (Just sleep with a few inputs and outputs.)
# INPUT input{...}.cel: input{...}.cel TYPE AFFY 
# OUTPUT input1.cel: input1.cel 
# OUTPUT input2.cel: input2.cel 
# OUTPUT input3.cel: input3.cel 
# OUTPUT input4.cel: input4.cel 

system("sleep 10")
